@@ -106,10 +106,12 @@ module Network.HTTP.Types
 , gatewayTimeout504
 , status505
 , httpVersionNotSupported505
--- * Headers
+  -- * Headers
+  -- ** Types
 , Header
 , RequestHeaders
 , ResponseHeaders
+  -- ** Common headers
 , headerAccept
 , headerAuthorization
 , headerCacheControl
@@ -118,6 +120,13 @@ module Network.HTTP.Types
 , headerContentType
 , headerContentMD5
 , headerDate
+  -- ** Byte ranges
+, ByteRange(..)
+, renderByteRangeBuilder
+, renderByteRange
+, ByteRanges
+, renderByteRangesBuilder
+, renderByteRanges
   -- * Query string
 , QueryItem
 , Query
@@ -138,6 +147,7 @@ module Network.HTTP.Types
   -- * Path segments
 , encodePathSegments
 , decodePathSegments
+, encodePathSegmentsRelative
   -- * Path (segments + query string)
 , encodePath
 , decodePath
@@ -190,20 +200,22 @@ module Network.HTTP.Types
 )
 where
 
-import           Control.Arrow            (second, (|||), (***))
+import           Control.Arrow                  (second, (|||), (***))
 import           Data.Array
-import           Data.Bits                (shiftL, (.|.))
+import           Data.Bits                      (shiftL, (.|.))
 import           Data.Char
 import           Data.Maybe
-import           Data.Monoid              (mempty, mappend, mconcat)
-import           Data.Text                (Text)
-import           Data.Text.Encoding       (encodeUtf8, decodeUtf8With)
-import           Data.Text.Encoding.Error (lenientDecode)
-import           Data.Word                (Word8)
-import qualified Blaze.ByteString.Builder as Blaze
-import qualified Data.ByteString          as B
-import qualified Data.ByteString.Char8    as B8
-import qualified Data.CaseInsensitive     as CI
+import           Data.Monoid                    (mempty, mappend, mconcat)
+import           Data.Text                      (Text)
+import           Data.Text.Encoding             (encodeUtf8, decodeUtf8With)
+import           Data.Text.Encoding.Error       (lenientDecode)
+import           Data.Word                      (Word8)
+import           Data.List                      (intersperse)
+import qualified Blaze.ByteString.Builder       as Blaze
+import qualified Blaze.ByteString.Builder.Char8 as Blaze
+import qualified Data.ByteString                as B
+import qualified Data.ByteString.Char8          as B8
+import qualified Data.CaseInsensitive           as CI
 
 type Ascii = B.ByteString
 
@@ -850,11 +862,37 @@ headerContentType   = (,) "Content-Type"
 headerContentMD5    = (,) "Content-MD5"
 headerDate          = (,) "Date"
 
+-- | RFC 2616 Byte range (individual). 
+-- 
+-- Negative indices are not allowed!
+data ByteRange 
+  = ByteRangeFrom !Integer
+  | ByteRangeFromTo !Integer !Integer
+  | ByteRangeSuffix !Integer
+
+renderByteRangeBuilder :: ByteRange -> Blaze.Builder
+renderByteRangeBuilder (ByteRangeFrom from) = Blaze.fromShow from `mappend` Blaze.fromChar '-'
+renderByteRangeBuilder (ByteRangeFromTo from to) = Blaze.fromShow from `mappend` Blaze.fromChar '-' `mappend` Blaze.fromShow to
+renderByteRangeBuilder (ByteRangeSuffix suffix) = Blaze.fromChar '-' `mappend` Blaze.fromShow suffix
+
+renderByteRange :: ByteRange -> Ascii
+renderByteRange = Blaze.toByteString . renderByteRangeBuilder
+
+-- | RFC 2616 Byte ranges (set).
+type ByteRanges = [ByteRange]
+
+renderByteRangesBuilder :: ByteRanges -> Blaze.Builder
+renderByteRangesBuilder xs = Blaze.copyByteString "bytes=" `mappend` 
+                             mconcat (intersperse (Blaze.fromChar ',') (map renderByteRangeBuilder xs))
+
+renderByteRanges :: ByteRanges -> Ascii
+renderByteRanges = Blaze.toByteString . renderByteRangesBuilder
+
 -- | Query item
 type QueryItem = (B.ByteString, Maybe B.ByteString)
 
 -- | Query.
---
+-- 
 -- General form: a=b&c=d, but if the value is Nothing, it becomes
 -- a&c=d.
 type Query = [QueryItem]
@@ -1026,21 +1064,21 @@ urlDecode replacePlus z = fst $ B.unfoldrN (B.length z) go z
 --
 -- * Performs percent encoding on all unreserved characters, as well as \:\@\=\+\$,
 --
--- * Intercalates with a slash.
+-- * Prepends each segment with a slash.
 --
 -- For example:
 --
--- > encodePathInfo [\"foo\", \"bar\", \"baz\"]
+-- > encodePathSegments [\"foo\", \"bar\", \"baz\"]
 --
--- \"foo\/bar\/baz\"
+-- \"\/foo\/bar\/baz\"
 --
--- > encodePathInfo [\"foo bar\", \"baz\/bin\"]
+-- > encodePathSegments [\"foo bar\", \"baz\/bin\"]
 --
--- \"foo\%20bar\/baz\%2Fbin\"
+-- \"\/foo\%20bar\/baz\%2Fbin\"
 --
--- > encodePathInfo [\"שלום\"]
+-- > encodePathSegments [\"שלום\"]
 --
--- \"%D7%A9%D7%9C%D7%95%D7%9D\"
+-- \"\/%D7%A9%D7%9C%D7%95%D7%9D\"
 --
 -- Huge thanks to Jeremy Shaw who created the original implementation of this
 -- function in web-routes and did such thorough research to determine all
@@ -1052,6 +1090,10 @@ encodePathSegments (x:xs) =
     `mappend` encodePathSegment x
     `mappend` encodePathSegments xs
 
+-- | Like encodePathSegments, but without the initial slash.
+encodePathSegmentsRelative :: [Text] -> Blaze.Builder
+encodePathSegmentsRelative xs = mconcat $ intersperse (Blaze.copyByteString "/") (map encodePathSegment xs)
+
 encodePathSegment :: Text -> Blaze.Builder
 encodePathSegment = urlEncodeBuilder False . encodeUtf8
 
